@@ -22,7 +22,7 @@ import os
 os.environ["OPENBLAS_NUM_THREADS"] = "1"
 os.environ["MKL_NUM_THREADS"] = "1"
 
-from nums.api import init, read, write, delete, read_csv
+from nums.api import init, read, write, delete, read_csv, from_modin
 from nums.core.version import __version__
 
-__all__ = ["numpy", "init", "read", "write", "delete", "read_csv"]
+__all__ = ["numpy", "init", "read", "write", "delete", "read_csv", "from_modin"]
